const fs = require('fs');

const operation = process.argv[2];
const input = process.argv[3];
const output = process.argv[4];

if (process.argv.length !== 5 || !['decode', 'encode'].includes(operation)) {
  console.error(`node index [encode|decode] <input> <output>
    encode - convert markdown to leanpub format
    decode - convert leanpub to markdown format
    input  - input file or directory
    output - output file or directory (will be overwritten)`)
	process.exit(1);
}

const convert = _getConverter(operation);
const write = _getWriter(input, output);

write(input, output, convert);

function _isDirectory(path) {
	return fs.statSync(path).isDirectory();
}

function _getConverter(operation) {
	const { convert } = operation === 'encode'
		? require('./lib/markdown-to-leanpub.converter')
		: require('./lib/leanpub-to-markdown.converter');

	return convert;
}

function _getWriter(input) {
  const { write } = !_isDirectory(input)
    ? require('./lib/file.writer')
    : require('./lib/recursive.writer');

	return write;
}
