function convert(input) {
  const output = input.replace(
    /^\{.*?lang="?(.*?)"?}\n~~~~~~~\n(.*?)~~~~~~~\n?/gms,
    (match, language, codeblock) => {
      const ranges = buildRanges(match.split('\n')).join(',');

      const header = '```' + language + (ranges ? `{${ranges}}` : '');
      let block = removeLeanpubComments(codeblock);
      const footer = '```\n';

      return [header, block.trim(), footer].join('\n');
    }
  );

  return removeLeanpubComments(output);
}

function stripLines(text, needles) {
  return needles.reduce(
    (output, needle) => output.split(needle + '\n').join(''),
    text
  );
}

// Assumes the "leanpub" comments are well-formatted and have pairs
function buildRanges(codeLines) {
  const ranges = [];

  let headerOffset = 0;
  let previousStart = null;

  codeLines.forEach((currentLine, index) => {
    if (currentLine.indexOf('# leanpub-start-insert') !== -1) {
      previousStart = index + 1;
      headerOffset += 1;
      return;
    }

    if (currentLine.indexOf('# leanpub-end-insert') !== -1) {
      headerOffset += 1;
      const range = buildRange(
        previousStart - headerOffset,
        index - 1 - headerOffset
      );
      ranges.push(range);
    }
  });

  return ranges;
}

function buildRange(from, to) {
  return from === to ? from : `${from}-${to}`;
}

function removeLeanpubComments(text) {
	return stripLines(text, ['# leanpub-start-insert', '# leanpub-end-insert']);
}

module.exports = { convert };
