function convert(text) {
  const input = text.split('\n');
  const codeblocks = parseCodeblocks(input);
  const wrapLines = getWrapLines(codeblocks);

  const output = [];

  for (let i = 0; i < input.length; i++) {
    const codeblock = codeblocks.find(codeblock => codeblock.line === i + 1);
    if (codeblock) {
      output.push(`{title="",lang="${codeblock.header}"}`);
      output.push('```');
    } else if (wrapLines.includes(i + 1)) {
      output.push('# leanpub-start-insert');
      output.push(input[i]);
      output.push('# leanpub-end-insert');
    } else {
      output.push(input[i]);
    }
  }

  return output
    .join('\n')
    .replace(/# leanpub-end-insert\n# leanpub-start-insert\n/gms, '')
    .replace(/```/g, '~~~~~~~');
}

function parseCodeblocks(input) {
  const codeblocks = [];

  for (let i = 0; i < input.length; i++) {
    const match = input[i].match(/```(\w+)(?:{(.*)})?/);
    if (!match) {
      continue;
    }

    const absoluteLineNumber = i + 1;
    const absoluteBlockEnd = getBlockEnd(input.slice(i)) + i;
    const range = match[2] ? parseRange(match[2]).map(n => i + 1 + n) : [];
    const validRange = range.filter(n => n <= absoluteBlockEnd);

    if (range.length !== validRange.length) {
      console.error(`Code block at line ${i + 1} has out of bounds references`);
    }

    codeblocks.push({
      line: i + 1,
      header: match[1],
      range: validRange
    });
  }

  return codeblocks;
}

function getBlockEnd(lines) {
  return lines.findIndex(line => /```\s*$/.test(line));
}

function getWrapLines(codeblocks) {
  return [
    ...new Set([].concat(...codeblocks.map(codeblock => codeblock.range)))
  ].sort();
}

function* range(from, to) {
  for (let i = from; i <= to; i++) {
    yield i;
  }
}

function parseRangeFrom(string) {
  let [from, to] = string.split('-');
  to = to === undefined ? from : to;

  return [...range(parseInt(from), parseInt(to))];
}

function parseRange(string) {
  const nestedRanges = string.split(',').map(range => parseRangeFrom(range));
  return [...new Set([].concat(...nestedRanges))].sort((a, b) => a - b);
}

module.exports = { convert };
