const fs = require('fs');
const path = require('path');

const { write: writeFile } = require('./file.writer');

function write(input, output, convert) {
  const relativePaths = _getFiles(input).map(file => path.relative(input, file));
  for (const file of relativePaths) {
    const inputFile = path.join(input, file);
    const outputFile = path.join(output, file);
    _ensureDirectoryExists(outputFile);

    if (_matchesExtension(file, '.md')) {
      writeFile(inputFile, outputFile, convert);
      continue;
    }

    console.log('Copying: ' + inputFile + ' -> ' + outputFile);
    fs.copyFileSync(inputFile, outputFile);
  }
}

function _matchesExtension(file, extension) {
  return file.substring(file.length - extension.length) === extension;
}

function _getFiles(directory) {
	return fs
		.readdirSync(directory)
		.map((file) => path.join(directory, file))
    .reduce(
      (files, current) => {
        return files.concat(fs.statSync(current).isDirectory() ? _getFiles(current) : [current]);
      }, []);
}

function _ensureDirectoryExists(file) {
  var dirname = path.dirname(file);

  if (fs.existsSync(dirname)) {
    return;
  }

  _ensureDirectoryExists(dirname);
  fs.mkdirSync(dirname);
}

module.exports = { write };
